/**
 * Copyright (c) 2014, 2015 Christian W. Damus and others.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Christian W. Damus - Initial API and implementation
 */
package org.eclipse.papyrus.uml.profile.elementtypesconfigurations.generator;

import com.google.common.collect.Iterables;
import java.util.ArrayList;
import java.util.HashMap;
import javax.inject.Inject;
import javax.inject.Singleton;
import org.eclipse.emf.common.util.EList;
import org.eclipse.papyrus.infra.elementtypesconfigurations.AdviceBindingConfiguration;
import org.eclipse.papyrus.infra.elementtypesconfigurations.ElementTypeConfiguration;
import org.eclipse.papyrus.infra.elementtypesconfigurations.ElementTypeSetConfiguration;
import org.eclipse.papyrus.infra.elementtypesconfigurations.ElementtypesconfigurationsFactory;
import org.eclipse.papyrus.infra.elementtypesconfigurations.SpecializationTypeConfiguration;
import org.eclipse.papyrus.uml.profile.elementtypesconfigurations.generator.ApplyStereotypeAdviceRule;
import org.eclipse.papyrus.uml.profile.elementtypesconfigurations.generator.ElementTypeRule;
import org.eclipse.papyrus.uml.profile.elementtypesconfigurations.generator.Identifiers;
import org.eclipse.papyrus.uml.profile.elementtypesconfigurations.generator.ImpliedExtension;
import org.eclipse.papyrus.uml.profile.elementtypesconfigurations.generator.UML;
import org.eclipse.papyrus.uml.profile.elementtypesconfigurations.generator.UMLElementTypes;
import org.eclipse.papyrus.uml.tools.elementtypesconfigurations.applystereotypeadviceconfiguration.ApplyStereotypeAdviceConfiguration;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Transformation rule for generating an {@link ElementTypeSetConfiguration} from a UML {@link Profile}.
 */
@Singleton
@SuppressWarnings("all")
public class ConfigurationSetRule {
  @Extension
  private static ElementtypesconfigurationsFactory elementtypesconfigurationsFactory = ElementtypesconfigurationsFactory.eINSTANCE;
  
  @Inject
  @Extension
  private Identifiers _identifiers;
  
  @Inject
  @Extension
  private UML _uML;
  
  @Inject
  @Extension
  private UMLElementTypes _uMLElementTypes;
  
  @Inject
  @Extension
  private ElementTypeRule _elementTypeRule;
  
  @Inject
  @Extension
  private ApplyStereotypeAdviceRule _applyStereotypeAdviceRule;
  
  public ElementTypeSetConfiguration toConfigurationSet(final Profile umlProfile) {
    final ArrayList<?> _cacheKey = CollectionLiterals.newArrayList(umlProfile);
    final ElementTypeSetConfiguration _result;
    synchronized (_createCache_toConfigurationSet) {
      if (_createCache_toConfigurationSet.containsKey(_cacheKey)) {
        return _createCache_toConfigurationSet.get(_cacheKey);
      }
      ElementTypeSetConfiguration _createElementTypeSetConfiguration = ConfigurationSetRule.elementtypesconfigurationsFactory.createElementTypeSetConfiguration();
      _result = _createElementTypeSetConfiguration;
      _createCache_toConfigurationSet.put(_cacheKey, _result);
    }
    _init_toConfigurationSet(_result, umlProfile);
    return _result;
  }
  
  private final HashMap<ArrayList<?>, ElementTypeSetConfiguration> _createCache_toConfigurationSet = CollectionLiterals.newHashMap();
  
  private void _init_toConfigurationSet(final ElementTypeSetConfiguration it, final Profile umlProfile) {
    this._identifiers.setIdentifierBase(umlProfile);
    String _qualified = this._identifiers.getQualified("elementTypes");
    it.setIdentifier(_qualified);
    String _name = umlProfile.getName();
    it.setName(_name);
    String _elvis = null;
    ElementTypeSetConfiguration _baseUMLElementTypeSet = this._uMLElementTypes.getBaseUMLElementTypeSet();
    String _metamodelNsURI = null;
    if (_baseUMLElementTypeSet!=null) {
      _metamodelNsURI=_baseUMLElementTypeSet.getMetamodelNsURI();
    }
    if (_metamodelNsURI != null) {
      _elvis = _metamodelNsURI;
    } else {
      _elvis = UMLPackage.eNS_URI;
    }
    it.setMetamodelNsURI(_elvis);
    Iterable<ImpliedExtension> _allExtensions = this._uML.getAllExtensions(umlProfile);
    final Procedure1<ImpliedExtension> _function = new Procedure1<ImpliedExtension>() {
      public void apply(final ImpliedExtension ext) {
        EList<ElementTypeConfiguration> _elementTypeConfigurations = it.getElementTypeConfigurations();
        org.eclipse.uml2.uml.Class _metaclass = ext.getMetaclass();
        Iterable<? extends ElementTypeConfiguration> _diagramSpecificElementTypes = ConfigurationSetRule.this._uMLElementTypes.getDiagramSpecificElementTypes(_metaclass);
        final Function1<ElementTypeConfiguration, SpecializationTypeConfiguration> _function = new Function1<ElementTypeConfiguration, SpecializationTypeConfiguration>() {
          public SpecializationTypeConfiguration apply(final ElementTypeConfiguration it) {
            return ConfigurationSetRule.this._elementTypeRule.toElementType(ext, it);
          }
        };
        Iterable<SpecializationTypeConfiguration> _map = IterableExtensions.map(_diagramSpecificElementTypes, _function);
        Iterables.<ElementTypeConfiguration>addAll(_elementTypeConfigurations, _map);
        org.eclipse.uml2.uml.Class _metaclass_1 = ext.getMetaclass();
        Iterable<? extends ElementTypeConfiguration> _diagramSpecificElementTypes_1 = ConfigurationSetRule.this._uMLElementTypes.getDiagramSpecificElementTypes(_metaclass_1);
        final Function1<ElementTypeConfiguration, Boolean> _function_1 = new Function1<ElementTypeConfiguration, Boolean>() {
          public Boolean apply(final ElementTypeConfiguration it) {
            boolean _hasSemanticSupertype = ConfigurationSetRule.this._uMLElementTypes.hasSemanticSupertype(it);
            return Boolean.valueOf((!_hasSemanticSupertype));
          }
        };
        final Iterable<? extends ElementTypeConfiguration> typesNeedingAdvice = IterableExtensions.filter(_diagramSpecificElementTypes_1, _function_1);
        EList<AdviceBindingConfiguration> _adviceBindingsConfigurations = it.getAdviceBindingsConfigurations();
        final Function1<ElementTypeConfiguration, ApplyStereotypeAdviceConfiguration> _function_2 = new Function1<ElementTypeConfiguration, ApplyStereotypeAdviceConfiguration>() {
          public ApplyStereotypeAdviceConfiguration apply(final ElementTypeConfiguration it) {
            Stereotype _stereotype = ext.getStereotype();
            return ConfigurationSetRule.this._applyStereotypeAdviceRule.toAdviceConfiguration(_stereotype, ext, it);
          }
        };
        Iterable<ApplyStereotypeAdviceConfiguration> _map_1 = IterableExtensions.map(typesNeedingAdvice, _function_2);
        Iterables.<AdviceBindingConfiguration>addAll(_adviceBindingsConfigurations, _map_1);
      }
    };
    IterableExtensions.<ImpliedExtension>forEach(_allExtensions, _function);
  }
}
