/**
 * Copyright (c) 2014, 2015 Christian W. Damus and others.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Christian W. Damus - Initial API and implementation
 */
package org.eclipse.papyrus.uml.profile.elementtypesconfigurations.generator;

import com.google.common.collect.Iterables;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import javax.inject.Singleton;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.papyrus.uml.profile.elementtypesconfigurations.generator.ImpliedExtension;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Namespace;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;

/**
 * Utility extensions for working with UML models and elements.
 */
@Singleton
@SuppressWarnings("all")
public class UML {
  private final Set<EClass> relationshipLikeUMLMetaclasses = Collections.<EClass>unmodifiableSet(CollectionLiterals.<EClass>newHashSet(UMLPackage.Literals.RELATIONSHIP, UMLPackage.Literals.GENERALIZATION_SET, UMLPackage.Literals.CONNECTOR, UMLPackage.Literals.ACTIVITY_EDGE, UMLPackage.Literals.TRANSITION, UMLPackage.Literals.MESSAGE));
  
  protected Profile _getProfile(final Profile element) {
    return element;
  }
  
  protected Profile _getProfile(final NamedElement element) {
    EList<org.eclipse.uml2.uml.Package> _allOwningPackages = element.allOwningPackages();
    Iterable<Profile> _filter = Iterables.<Profile>filter(_allOwningPackages, Profile.class);
    return IterableExtensions.<Profile>head(_filter);
  }
  
  protected Profile _getProfile(final ImpliedExtension umlExtension) {
    Stereotype _stereotype = umlExtension.getStereotype();
    return _stereotype.getProfile();
  }
  
  protected Profile _getRootProfile(final Profile element) {
    Profile _elvis = null;
    Namespace _namespace = element.getNamespace();
    Profile _rootProfile = null;
    if (_namespace!=null) {
      _rootProfile=this.getRootProfile(_namespace);
    }
    if (_rootProfile != null) {
      _elvis = _rootProfile;
    } else {
      _elvis = element;
    }
    return _elvis;
  }
  
  protected Profile _getRootProfile(final NamedElement element) {
    EList<org.eclipse.uml2.uml.Package> _allOwningPackages = element.allOwningPackages();
    Iterable<Profile> _filter = Iterables.<Profile>filter(_allOwningPackages, Profile.class);
    return IterableExtensions.<Profile>last(_filter);
  }
  
  public Iterable<ImpliedExtension> getAllExtensions(final org.eclipse.uml2.uml.Package package_) {
    EList<Type> _ownedTypes = package_.getOwnedTypes();
    Iterable<Stereotype> _filter = Iterables.<Stereotype>filter(_ownedTypes, Stereotype.class);
    final Function1<Stereotype, Iterable<ImpliedExtension>> _function = new Function1<Stereotype, Iterable<ImpliedExtension>>() {
      public Iterable<ImpliedExtension> apply(final Stereotype it) {
        return UML.this.impliedExtensions(it);
      }
    };
    Iterable<Iterable<ImpliedExtension>> _map = IterableExtensions.<Stereotype, Iterable<ImpliedExtension>>map(_filter, _function);
    Iterable<ImpliedExtension> _flatten = Iterables.<ImpliedExtension>concat(_map);
    EList<org.eclipse.uml2.uml.Package> _nestedPackages = package_.getNestedPackages();
    final Function1<org.eclipse.uml2.uml.Package, Iterable<ImpliedExtension>> _function_1 = new Function1<org.eclipse.uml2.uml.Package, Iterable<ImpliedExtension>>() {
      public Iterable<ImpliedExtension> apply(final org.eclipse.uml2.uml.Package it) {
        return UML.this.getAllExtensions(it);
      }
    };
    List<Iterable<ImpliedExtension>> _map_1 = ListExtensions.<org.eclipse.uml2.uml.Package, Iterable<ImpliedExtension>>map(_nestedPackages, _function_1);
    Iterable<ImpliedExtension> _flatten_1 = Iterables.<ImpliedExtension>concat(_map_1);
    return Iterables.<ImpliedExtension>concat(_flatten, _flatten_1);
  }
  
  public Iterable<ImpliedExtension> impliedExtensions(final Stereotype stereotype) {
    EList<org.eclipse.uml2.uml.Class> _allExtendedMetaclasses = stereotype.getAllExtendedMetaclasses();
    final Function1<org.eclipse.uml2.uml.Class, ImpliedExtension> _function = new Function1<org.eclipse.uml2.uml.Class, ImpliedExtension>() {
      public ImpliedExtension apply(final org.eclipse.uml2.uml.Class it) {
        return new ImpliedExtension(stereotype, it);
      }
    };
    return ListExtensions.<org.eclipse.uml2.uml.Class, ImpliedExtension>map(_allExtendedMetaclasses, _function);
  }
  
  public boolean isRelationship(final EClass eClass) {
    final Function1<EClass, Boolean> _function = new Function1<EClass, Boolean>() {
      public Boolean apply(final EClass it) {
        return Boolean.valueOf(it.isSuperTypeOf(eClass));
      }
    };
    return IterableExtensions.<EClass>exists(this.relationshipLikeUMLMetaclasses, _function);
  }
  
  public Iterable<EReference> getSourceReferences(final EClass relationshipClass) {
    Iterable<EReference> _switchResult = null;
    boolean _matched = false;
    if (!_matched) {
      boolean _isSuperTypeOf = UMLPackage.Literals.DIRECTED_RELATIONSHIP.isSuperTypeOf(relationshipClass);
      if (_isSuperTypeOf) {
        _matched=true;
        EList<EReference> _eAllReferences = relationshipClass.getEAllReferences();
        final Function1<EReference, Boolean> _function = new Function1<EReference, Boolean>() {
          public Boolean apply(final EReference it) {
            boolean _and = false;
            boolean _and_1 = false;
            boolean _isDerived = it.isDerived();
            boolean _not = (!_isDerived);
            if (!_not) {
              _and_1 = false;
            } else {
              boolean _isChangeable = it.isChangeable();
              _and_1 = _isChangeable;
            }
            if (!_and_1) {
              _and = false;
            } else {
              boolean _subsets = UML.this.subsets(it, UMLPackage.Literals.DIRECTED_RELATIONSHIP__SOURCE);
              _and = _subsets;
            }
            return Boolean.valueOf(_and);
          }
        };
        _switchResult = IterableExtensions.<EReference>filter(_eAllReferences, _function);
      }
    }
    if (!_matched) {
      boolean _isSuperTypeOf_1 = UMLPackage.Literals.ASSOCIATION.isSuperTypeOf(relationshipClass);
      if (_isSuperTypeOf_1) {
        _matched=true;
        _switchResult = Collections.<EReference>unmodifiableList(CollectionLiterals.<EReference>newArrayList(UMLPackage.Literals.ASSOCIATION__END_TYPE));
      }
    }
    if (!_matched) {
      boolean _isSuperTypeOf_2 = UMLPackage.Literals.CONNECTOR.isSuperTypeOf(relationshipClass);
      if (_isSuperTypeOf_2) {
        _matched=true;
        _switchResult = Collections.<EReference>unmodifiableList(CollectionLiterals.<EReference>newArrayList(UMLPackage.Literals.CONNECTOR__END));
      }
    }
    if (!_matched) {
      boolean _isSuperTypeOf_3 = UMLPackage.Literals.ACTIVITY_EDGE.isSuperTypeOf(relationshipClass);
      if (_isSuperTypeOf_3) {
        _matched=true;
        _switchResult = Collections.<EReference>unmodifiableList(CollectionLiterals.<EReference>newArrayList(UMLPackage.Literals.ACTIVITY_EDGE__SOURCE));
      }
    }
    if (!_matched) {
      boolean _isSuperTypeOf_4 = UMLPackage.Literals.TRANSITION.isSuperTypeOf(relationshipClass);
      if (_isSuperTypeOf_4) {
        _matched=true;
        _switchResult = Collections.<EReference>unmodifiableList(CollectionLiterals.<EReference>newArrayList(UMLPackage.Literals.TRANSITION__SOURCE));
      }
    }
    if (!_matched) {
      boolean _isSuperTypeOf_5 = UMLPackage.Literals.MESSAGE.isSuperTypeOf(relationshipClass);
      if (_isSuperTypeOf_5) {
        _matched=true;
        _switchResult = Collections.<EReference>unmodifiableList(CollectionLiterals.<EReference>newArrayList(UMLPackage.Literals.MESSAGE__SEND_EVENT));
      }
    }
    if (!_matched) {
      boolean _isSuperTypeOf_6 = UMLPackage.Literals.GENERALIZATION_SET.isSuperTypeOf(relationshipClass);
      if (_isSuperTypeOf_6) {
        _matched=true;
        _switchResult = Collections.<EReference>unmodifiableList(CollectionLiterals.<EReference>newArrayList(UMLPackage.Literals.GENERALIZATION_SET__GENERALIZATION));
      }
    }
    if (!_matched) {
      _switchResult = Collections.<EReference>unmodifiableList(CollectionLiterals.<EReference>newArrayList());
    }
    return _switchResult;
  }
  
  public Iterable<EReference> getTargetReferences(final EClass relationshipClass) {
    Iterable<EReference> _switchResult = null;
    boolean _matched = false;
    if (!_matched) {
      boolean _isSuperTypeOf = UMLPackage.Literals.DIRECTED_RELATIONSHIP.isSuperTypeOf(relationshipClass);
      if (_isSuperTypeOf) {
        _matched=true;
        EList<EReference> _eAllReferences = relationshipClass.getEAllReferences();
        final Function1<EReference, Boolean> _function = new Function1<EReference, Boolean>() {
          public Boolean apply(final EReference it) {
            boolean _and = false;
            boolean _and_1 = false;
            boolean _isDerived = it.isDerived();
            boolean _not = (!_isDerived);
            if (!_not) {
              _and_1 = false;
            } else {
              boolean _isChangeable = it.isChangeable();
              _and_1 = _isChangeable;
            }
            if (!_and_1) {
              _and = false;
            } else {
              boolean _subsets = UML.this.subsets(it, UMLPackage.Literals.DIRECTED_RELATIONSHIP__TARGET);
              _and = _subsets;
            }
            return Boolean.valueOf(_and);
          }
        };
        _switchResult = IterableExtensions.<EReference>filter(_eAllReferences, _function);
      }
    }
    if (!_matched) {
      boolean _isSuperTypeOf_1 = UMLPackage.Literals.ASSOCIATION.isSuperTypeOf(relationshipClass);
      if (_isSuperTypeOf_1) {
        _matched=true;
        _switchResult = Collections.<EReference>unmodifiableList(CollectionLiterals.<EReference>newArrayList(UMLPackage.Literals.ASSOCIATION__END_TYPE));
      }
    }
    if (!_matched) {
      boolean _isSuperTypeOf_2 = UMLPackage.Literals.CONNECTOR.isSuperTypeOf(relationshipClass);
      if (_isSuperTypeOf_2) {
        _matched=true;
        _switchResult = Collections.<EReference>unmodifiableList(CollectionLiterals.<EReference>newArrayList(UMLPackage.Literals.CONNECTOR__END));
      }
    }
    if (!_matched) {
      boolean _isSuperTypeOf_3 = UMLPackage.Literals.ACTIVITY_EDGE.isSuperTypeOf(relationshipClass);
      if (_isSuperTypeOf_3) {
        _matched=true;
        _switchResult = Collections.<EReference>unmodifiableList(CollectionLiterals.<EReference>newArrayList(UMLPackage.Literals.ACTIVITY_EDGE__TARGET));
      }
    }
    if (!_matched) {
      boolean _isSuperTypeOf_4 = UMLPackage.Literals.TRANSITION.isSuperTypeOf(relationshipClass);
      if (_isSuperTypeOf_4) {
        _matched=true;
        _switchResult = Collections.<EReference>unmodifiableList(CollectionLiterals.<EReference>newArrayList(UMLPackage.Literals.TRANSITION__TARGET));
      }
    }
    if (!_matched) {
      boolean _isSuperTypeOf_5 = UMLPackage.Literals.MESSAGE.isSuperTypeOf(relationshipClass);
      if (_isSuperTypeOf_5) {
        _matched=true;
        _switchResult = Collections.<EReference>unmodifiableList(CollectionLiterals.<EReference>newArrayList(UMLPackage.Literals.MESSAGE__RECEIVE_EVENT));
      }
    }
    if (!_matched) {
      boolean _isSuperTypeOf_6 = UMLPackage.Literals.GENERALIZATION_SET.isSuperTypeOf(relationshipClass);
      if (_isSuperTypeOf_6) {
        _matched=true;
        _switchResult = Collections.<EReference>unmodifiableList(CollectionLiterals.<EReference>newArrayList(UMLPackage.Literals.GENERALIZATION_SET__GENERALIZATION));
      }
    }
    if (!_matched) {
      _switchResult = Collections.<EReference>unmodifiableList(CollectionLiterals.<EReference>newArrayList());
    }
    return _switchResult;
  }
  
  public boolean subsets(final EReference subset, final EReference superset) {
    EAnnotation _eAnnotation = subset.getEAnnotation("subsets");
    EList<EObject> _references = null;
    if (_eAnnotation!=null) {
      _references=_eAnnotation.getReferences();
    }
    boolean _contains = false;
    if (_references!=null) {
      _contains=_references.contains(superset);
    }
    return _contains;
  }
  
  public Profile getProfile(final Object element) {
    if (element instanceof Profile) {
      return _getProfile((Profile)element);
    } else if (element instanceof NamedElement) {
      return _getProfile((NamedElement)element);
    } else if (element instanceof ImpliedExtension) {
      return _getProfile((ImpliedExtension)element);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(element).toString());
    }
  }
  
  public Profile getRootProfile(final NamedElement element) {
    if (element instanceof Profile) {
      return _getRootProfile((Profile)element);
    } else if (element != null) {
      return _getRootProfile(element);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(element).toString());
    }
  }
}
